#ifndef _COLOR_H_
#define _COLOR_H_

/// A class representing a 4-channel RGBA color. Channel values range from 0.0f to 1.0f.
class Color
{
	public:
		
		/// Create a default color (opaque black).
		inline Color()
			:	r( 0.0f ),
				g( 0.0f ),
				b( 0.0f ),
				a( 1.0f )
		{
		}
		
		
		/// Create a fully opaque color.
		inline Color( float newR, float newG, float newB )
			:	r( newR ),
				g( newG ),
				b( newB ),
				a( 1.0f )
		{
		}
		
		
		/// Create a color from red, green, blue, and alpha channels.
		inline Color( float newR, float newG, float newB, float newA )
			:	r( newR ),
				g( newG ),
				b( newB ),
				a( newA )
		{
		}
		
		friend Color operator*(Color c, double f);
		friend Color operator*(Color c1, Color c2);
		friend Color operator+(Color c1, Color c2);
		
		/// Color components. 1.0f == channel at full, 0.0f == channel at 0%.
		float r, g, b, a;
		
		
};

//Added by Ben Newton
inline Color operator*(Color c1, Color c2)
{
	return Color(c1.r*c2.r, c1.g*c2.g, c1.b*c2.b, c1.a*c2.a);
}

//Added by Ben Newton
inline Color operator*(Color c, double f)
{
	return Color(c.r*f, c.g*f, c.b*f, c.a*f);
}

//Added by Ben Newton
inline Color operator+(Color c1, Color c2)
{
	return Color(c1.r+c2.r, c1.g+c2.g, c1.b+c2.b, c1.a+c2.a);
}

#endif //_COLOR_H_