#ifndef _MATRIX_H_
#define _MATRIX_H_

#include "Vector3.h"

//implements a 3x3 matrix
class Matrix
{
public:
	inline Matrix()
	{
		//initialize to identity?
		v[0][0] = 1.0;    //0,0 0,1 0,2
		v[0][1] = 0.0;    //1,0 1,1 1,2
		v[0][2] = 0.0;    //2,0 2,1 2,2
		v[1][0] = 0.0;
		v[1][1] = 1.0;
		v[1][2] = 0.0;
		v[2][0] = 0.0;
		v[2][1] = 0.0;
		v[2][2] = 1.0;
	}

	inline Matrix(float v00, float v01, float v02, 
		float v10, float v11, float v12, 
		float v20, float v21, float v22)
	{
		v[0][0] = v00;    //0,0 0,1 0,2
		v[0][1] = v01;    //1,0 1,1 1,2
		v[0][2] = v02;    //2,0 2,1 2,2
		v[1][0] = v10;
		v[1][1] = v11;
		v[1][2] = v12;
		v[2][0] = v20;
		v[2][1] = v21;
		v[2][2] = v22;
	}
	friend Matrix operator*(const Matrix& a, const Matrix& b);
	friend Vector3 operator*(const Matrix& m, const Vector3& v);
float v[3][3]; 
};

inline Matrix operator*(const Matrix& a, const Matrix& b)
{
	return Matrix(
		a.v[0][0]*b.v[0][0] + a.v[0][1]*b.v[1][0] + a.v[0][2]*b.v[2][0],
		a.v[0][0]*b.v[0][1] + a.v[0][1]*b.v[1][1] + a.v[0][2]*b.v[2][1],
		a.v[0][0]*b.v[0][2] + a.v[0][1]*b.v[1][2] + a.v[0][2]*b.v[2][2],
		a.v[1][0]*b.v[0][0] + a.v[1][1]*b.v[1][0] + a.v[1][2]*b.v[2][0],
		a.v[1][0]*b.v[0][1] + a.v[1][1]*b.v[1][1] + a.v[1][2]*b.v[2][1],
		a.v[1][0]*b.v[0][2] + a.v[1][1]*b.v[1][2] + a.v[1][2]*b.v[2][2],
		a.v[2][0]*b.v[0][0] + a.v[2][1]*b.v[1][0] + a.v[2][2]*b.v[2][0],
		a.v[2][0]*b.v[0][1] + a.v[2][1]*b.v[1][1] + a.v[2][2]*b.v[2][1],
		a.v[2][0]*b.v[0][2] + a.v[2][1]*b.v[1][2] + a.v[2][2]*b.v[2][2]);
}

inline Vector3 operator*(const Matrix& m, const Vector3& v)
{
	return Vector3(
		m.v[0][0]*v.e[0] + m.v[0][1]*v.e[1] + m.v[0][2]*v.e[2],
		m.v[1][0]*v.e[0] + m.v[1][1]*v.e[1] + m.v[1][2]*v.e[2],
		m.v[2][0]*v.e[0] + m.v[2][1]*v.e[1] + m.v[2][2]*v.e[2]);
}

#endif //_MATRIX_H_