#ifndef _MATRIX4_H_
#define _MATRIX4_H_

#include "Vector3.h"

//implements a 4x4 matrix
class Matrix4
{
public:
	inline Matrix4()
	{
		//initialize to identity?
		v[0][0] = 1.0;    //0,0 0,1 0,2 0,3
		v[0][1] = 0.0;    //1,0 1,1 1,2 1,3
		v[0][2] = 0.0;
		v[0][3] = 0.0;    //2,0 2,1 2,2 2,3
		v[1][0] = 0.0;    //3,0 3,1 3,2 3,3
		v[1][1] = 1.0;
		v[1][2] = 0.0;
		v[1][3] = 0.0;
		v[2][0] = 0.0;
		v[2][1] = 0.0;
		v[2][2] = 1.0;
		v[2][3] = 0.0;
		v[3][0] = 0.0;
		v[3][1] = 0.0;
		v[3][2] = 0.0;
		v[3][3] = 1.0;
	}

	inline Matrix4(float v00, float v01, float v02, float v03, 
		float v10, float v11, float v12, float v13,
		float v20, float v21, float v22, float v23,
		float v30, float v31, float v32, float v33)
	{
		v[0][0] = v00;    //0,0 0,1 0,2 0,3
		v[0][1] = v01;    //1,0 1,1 1,2 1,3
		v[0][2] = v02;    //2,0 2,1 2,2 2,3
		v[0][3] = v03;
		v[1][0] = v10;    //3,0 3,1 3,2 3,3
		v[1][1] = v11;
		v[1][2] = v12;
		v[1][3] = v13;
		v[2][0] = v20;
		v[2][1] = v21;
		v[2][2] = v22;
		v[2][3] = v23;
		v[3][0] = v30;
		v[3][1] = v31;
		v[3][2] = v32;
		v[3][3] = v33;
	}
	friend Matrix4 operator*(const Matrix4& a, const Matrix4& b);
	friend Vector3 operator*(const Matrix4& m, const Vector3& v);
float v[4][4]; 
};

inline Matrix4 operator*(const Matrix4& a, const Matrix4& b)
{
	return Matrix4(
		a.v[0][0]*b.v[0][0] + a.v[0][1]*b.v[1][0] + a.v[0][2]*b.v[2][0] + a.v[0][3]*b.v[3][0],
		a.v[0][0]*b.v[0][1] + a.v[0][1]*b.v[1][1] + a.v[0][2]*b.v[2][1] + a.v[0][3]*b.v[3][1],
		a.v[0][0]*b.v[0][2] + a.v[0][1]*b.v[1][2] + a.v[0][2]*b.v[2][2] + a.v[0][3]*b.v[3][2],
		a.v[0][0]*b.v[0][3] + a.v[0][1]*b.v[1][3] + a.v[0][2]*b.v[2][3] + a.v[0][3]*b.v[3][3],
		
		a.v[1][0]*b.v[0][0] + a.v[1][1]*b.v[1][0] + a.v[1][2]*b.v[2][0] + a.v[1][3]*b.v[3][0],
		a.v[1][0]*b.v[0][1] + a.v[1][1]*b.v[1][1] + a.v[1][2]*b.v[2][1] + a.v[1][3]*b.v[3][1],
		a.v[1][0]*b.v[0][2] + a.v[1][1]*b.v[1][2] + a.v[1][2]*b.v[2][2] + a.v[1][3]*b.v[3][2],
		a.v[1][0]*b.v[0][3] + a.v[1][1]*b.v[1][3] + a.v[1][2]*b.v[2][3] + a.v[1][3]*b.v[3][3],
		
		a.v[2][0]*b.v[0][0] + a.v[2][1]*b.v[1][0] + a.v[2][2]*b.v[2][0] + a.v[2][3]*b.v[3][0],
		a.v[2][0]*b.v[0][1] + a.v[2][1]*b.v[1][1] + a.v[2][2]*b.v[2][1] + a.v[2][3]*b.v[3][1],
		a.v[2][0]*b.v[0][2] + a.v[2][1]*b.v[1][2] + a.v[2][2]*b.v[2][2] + a.v[2][3]*b.v[3][2],
		a.v[2][0]*b.v[0][3] + a.v[2][1]*b.v[1][3] + a.v[2][2]*b.v[2][3] + a.v[2][3]*b.v[3][3],
		
		a.v[3][0]*b.v[0][0] + a.v[3][1]*b.v[1][0] + a.v[3][2]*b.v[2][0] + a.v[3][3]*b.v[3][0],
		a.v[3][0]*b.v[0][1] + a.v[3][1]*b.v[1][1] + a.v[3][2]*b.v[2][1] + a.v[3][3]*b.v[3][1],
		a.v[3][0]*b.v[0][2] + a.v[3][1]*b.v[1][2] + a.v[3][2]*b.v[2][2] + a.v[3][3]*b.v[3][2],
		a.v[3][0]*b.v[0][3] + a.v[3][1]*b.v[1][3] + a.v[3][2]*b.v[2][3] + a.v[3][3]*b.v[3][3]);
}

inline Vector3 operator*(const Matrix4& m, const Vector3& v)
{
	return Vector3(
		m.v[0][0]*v.e[0] + m.v[0][1]*v.e[1] + m.v[0][2]*v.e[2] + m.v[0][3], //assume 1.0 last element of vec for now.
		m.v[1][0]*v.e[0] + m.v[1][1]*v.e[1] + m.v[1][2]*v.e[2] + m.v[1][3], //assume 1.0 for now.
  		m.v[2][0]*v.e[0] + m.v[2][1]*v.e[1] + m.v[2][2]*v.e[2] + m.v[2][3]); //assume 1.0 for now.
		//m.v[3][0]*v.e[0] + m.v[3][1]*v.e[1] + m.v[3][2]*v.e[2] + m.v[3][3]); //assume 1.0 for now.
}

#endif //_MATRIX4_H_