#ifndef _VECTOR3_H_
#define _VECTOR3_H_

#include <math.h>
#include <limits.h>
#include <stdlib.h>
#include <assert.h>
#include <iostream>
#include <iomanip>
#include <vector>

#include "MathDefines.h"

//may need to move below to common.h
//#if COMMON_COMPILER == COMPILER_MSVC 
//#   if COMMON_COMP_VER >= 1200
//#		define _CRT_SECURE_NO_DEPRECATE
//#       define FORCEINLINE __forceinline
///#   endif
//#else
#   define FORCEINLINE __inline
//#endif

/**
 *	3D Vector Class.
 *
 */

class Vector3  {
public:
    
    Vector3() { e[0] = 0; e[1] = 0; e[2] = 0;}
    Vector3(float e0, float e1, float e2) {e[0]=e0; e[1]=e1; e[2]=e2;}
	Vector3(float enew[3]) {e[0]=enew[0]; e[1]=enew[1]; e[2]=enew[2];}
    float x() const { return e[0]; }
    float y() const { return e[1]; }
    float z() const { return e[2]; }
    void setX(const float a) { e[0] = a; }
    void setY(const float a) { e[1] = a; }
    void setZ(const float a) { e[2] = a; }
	void set(const float *enew) {e[0]=enew[0]; e[1]=enew[1]; e[2]=enew[2]; }

    inline Vector3(const Vector3 &v) {
         e[0] = v.e[0]; e[1] = v.e[1]; e[2] = v.e[2];
    }

    const Vector3& operator+() const { return *this; }
    FORCEINLINE Vector3 operator-() const { return Vector3(-e[0], -e[1], -e[2]); }
    FORCEINLINE float& operator[](int i) {  return e[i]; }
    FORCEINLINE float operator[](int i) const { return e[i];}

    Vector3& operator+=(const Vector3 &v2);
    Vector3& operator-=(const Vector3 &v2);
    Vector3& operator*=(const float t);
    Vector3& operator/=(const float t);

    float length() const { return sqrt(e[0]*e[0] + e[1]*e[1] + e[2]*e[2]); }
    float squaredLength() const { return e[0]*e[0] + e[1]*e[1] + e[2]*e[2]; }

    void makeUnitVector();
	void randomize();
	void hemiSphereScatter(Vector3 &n);

    float minComponent() const { return e[indexOfMinComponent()]; }
    float maxComponent() const { return e[indexOfMaxComponent()]; }
    float maxAbsComponent() const { return e[indexOfMaxAbsComponent()]; }
    float minAbsComponent() const { return e[indexOfMinAbsComponent()]; }
    int indexOfMinComponent() const { 
	return (e[0]< e[1] && e[0]< e[2]) ? 0 : (e[1] < e[2] ? 1 : 2);
    }

    int indexOfMinAbsComponent() const {
	if (fabs(e[0]) < fabs(e[1]) && fabs(e[0]) < fabs(e[2]))
	    return 0;
	else if (fabs(e[1]) < fabs(e[2]))
	    return 1;
	else
	    return 2;
    }

    int indexOfMaxComponent() const {
	return (e[0]> e[1] && e[0]> e[2]) ? 0 : (e[1] > e[2] ? 1 : 2);
    }

    int indexOfMaxAbsComponent() const {
	if (fabs(e[0]) > fabs(e[1]) && fabs(e[0]) > fabs(e[2]))
	    return 0;
	else if (fabs(e[1]) > fabs(e[2]))
	    return 1;
	else
	    return 2;
    }

	//Anish Chandak 10/14/06
	float* getarray() {return e;}
	//Anish Chandak -- end
    float e[3];
};

// sungeui start -----------
// make _Vector4 for out-of-core accessing vector3 data and 
// this is just temporary solution; this class is just for minimizing coding effort

class _Vector4: public Vector3
{
	public:
	float m_alpha;			// it's dummy now

	_Vector4(Vector3 &v) {
		e[0] = v.e[0];
		e[1] = v.e[1];
		e[2] = v.e[2];
	}

	_Vector4() {
		e[0] = .0f;
		e[1] = .0f;
		e[2] = .0f;
	}
};
#define CONVERT_V3_to_V4(s, d) {d.e[0]=s.e[0]; d.e[1]=s.e[1]; d.e[2]=s.e[2];}
#define CONVERT_V4_to_V3(s, d) {d.e[0]=s.e[0]; d.e[1]=s.e[1]; d.e[2]=s.e[2];}
// sungeui end -----------------


inline bool operator==(const Vector3 &t1, const Vector3 &t2) {
   return ((t1[0]==t2[0])&&(t1[1]==t2[1])&&(t1[2]==t2[2]));
}

inline bool operator==(const _Vector4 &t1, const _Vector4 &t2) {
   return ((t1[0]==t2[0])&&(t1[1]==t2[1])&&(t1[2]==t2[2]));
}

inline bool operator!=(const Vector3 &t1, const Vector3 &t2) {
   return ((t1[0]!=t2[0])||(t1[1]!=t2[1])||(t1[2]!=t2[2]));
}

inline std::istream &operator>>(std::istream &is, Vector3 &t) {
   is >> t[0] >> t[1] >> t[2];
   return is;
}

inline std::ostream &operator<<(std::ostream &os, const Vector3 &t) {
   os << t[0] << " " << t[1] << " " << t[2];
   return os;
}

inline Vector3 unitVector(const Vector3& v) {
    float k = 1.0f / sqrt(v.e[0]*v.e[0] + v.e[1]*v.e[1] + v.e[2]*v.e[2]);
    return Vector3(v.e[0]*k, v.e[1]*k, v.e[2]*k);
}

inline void Vector3::makeUnitVector() {
    float k = 1.0f / sqrt(e[0]*e[0] + e[1]*e[1] + e[2]*e[2]);
    e[0] *= k; e[1] *= k; e[2] *= k;
}

/*inline void Vector3::randomize() {
	float r1 = (float)rand()/RAND_MAX*2-1;
	float r2 = (float)rand()/RAND_MAX;
	float r = sqrtf(1.0f - r1*r1);
	float p = 2 * PI * r2;
	
	e[0] = cosf(p)*r;
	e[1] = sinf(p)*r;
	e[2] = r1;

	return;

	do {
		e[0] = (float)rand()/RAND_MAX*2-1;
		e[1] = (float)rand()/RAND_MAX*2-1;
		e[2] = (float)rand()/RAND_MAX*2-1;
	} while ( this->squaredLength() > 1.0f);
}*/

inline Vector3 operator+(const Vector3 &v1, const Vector3 &v2) {
    return Vector3( v1.e[0] + v2.e[0], v1.e[1] + v2.e[1], v1.e[2] + v2.e[2]);
}

inline Vector3 operator-(const Vector3 &v1, const Vector3 &v2) {
    return Vector3( v1.e[0] - v2.e[0], v1.e[1] - v2.e[1], v1.e[2] - v2.e[2]);
}

inline Vector3 operator*(float t, const Vector3 &v) {
    return Vector3(t*v.e[0], t*v.e[1], t*v.e[2]); 
}

inline Vector3 operator*(const Vector3 &v, float t) {
    return Vector3(t*v.e[0], t*v.e[1], t*v.e[2]); 
}

inline Vector3 operator/(const Vector3 &v, float t) {
    return Vector3(v.e[0]/t, v.e[1]/t, v.e[2]/t); 
}


inline float dot(const Vector3 &v1, const Vector3 &v2) {
    return v1.e[0]*v2.e[0] + v1.e[1]*v2.e[1]  + v1.e[2]*v2.e[2];
}

inline float dot(const Vector3 &v1, const float *v2) {
	return v1.e[0]*v2[0] + v1.e[1]*v2[1]  + v1.e[2]*v2[2];
}

inline float dot(const float *v1, const float *v2) {
	return v1[0]*v2[0] + v1[1]*v2[1]  + v1[2]*v2[2];
}

inline void Vector3::hemiSphereScatter(Vector3 &n) {
	this->randomize();
	if( dot(n, *this) < 0 )
		*this = -*this;
}

inline float angleBetween(const Vector3 &v1, const Vector3 &v2) {
	Vector3 tv1 = Vector3(v1);
	Vector3 tv2 = Vector3(v2);
	tv1.makeUnitVector();
	tv2.makeUnitVector();
	float angle = dot(tv1, tv2);
	if(angle > 1.0f)
		return 0.0f;
	if(angle < -1.0f)
		return PI;
	angle = acos(angle);
	return angle;
}

inline Vector3 cross(const Vector3 v1, const Vector3 v2) {
    return Vector3( (v1.e[1]*v2.e[2] - v1.e[2]*v2.e[1]),
                      (v1.e[2]*v2.e[0] - v1.e[0]*v2.e[2]),
                      (v1.e[0]*v2.e[1] - v1.e[1]*v2.e[0]));
}

inline Vector3 projectToPlane(Vector3 v, Vector3 norm) {
	Vector3 vt = Vector3(v);
	Vector3 nt = Vector3(norm);
	//vt.makeUnitVector();
	//nt.makeUnitVector();
	float projAmount = dot(vt, nt);
	vt = vt - projAmount * nt;
	//vt.makeUnitVector();
	return vt;
}

inline bool operator<(const Vector3 &v1, const Vector3 &v2) {
	return (FLOAT_LT(v1.e[0], v2.e[0]) && FLOAT_LT(v1.e[1], v2.e[1]) && FLOAT_LT(v1.e[2], v2.e[2]));
}

inline bool operator<=(const Vector3 &v1, const Vector3 &v2) {
	return (FLOAT_LE(v1.e[0], v2.e[0]) && FLOAT_LE(v1.e[1], v2.e[1]) && FLOAT_LE(v1.e[2], v2.e[2]));
}

inline bool operator>(const Vector3 &v1, const Vector3 &v2) {
	return (FLOAT_GT(v1.e[0], v2.e[0]) && FLOAT_GT(v1.e[1], v2.e[1]) && FLOAT_GT(v1.e[2], v2.e[2]));
}

inline bool operator>=(const Vector3 &v1, const Vector3 &v2) {
	return (FLOAT_GE(v1.e[0], v2.e[0]) && FLOAT_GE(v1.e[1], v2.e[1]) && FLOAT_GE(v1.e[2], v2.e[2]));	
}

inline Vector3& Vector3::operator+=(const Vector3 &v){
    e[0]  += v.e[0];
    e[1]  += v.e[1];
    e[2]  += v.e[2];
    return *this;
}

inline Vector3& Vector3::operator-=(const Vector3& v) {
    e[0]  -= v.e[0];
    e[1]  -= v.e[1];
    e[2]  -= v.e[2];
    return *this;
}

inline Vector3& Vector3::operator*=(const float t) {
    e[0]  *= t;
    e[1]  *= t;
    e[2]  *= t;
    return *this;
}

inline Vector3& Vector3::operator/=(const float t) {
    e[0]  /= t;
    e[1]  /= t;
    e[2]  /= t;
    return *this;
}

inline
Vector3 reflect(const Vector3& in, const Vector3& normal)
{
  // assumes unit length normal

#define bothSideReflect
#ifdef bothSideReflect
	if(dot(in, normal) > 0)
		return in + normal * (2 * dot(in, -normal));

#endif
  return in - normal * (2 * dot(in, normal));
}

inline
Vector3 lambertian(const Vector3& in, const Vector3& normal)
{
  // assumes unit length normal
	Vector3 ref = reflect(in, normal);
	Vector3 random;
	random.hemiSphereScatter(ref);
	float r = (float)rand()/RAND_MAX*2-1;
  return in - normal * (2 * dot(in, normal));
}

// Area of a triangle:
inline float triangleArea(Vector3 v1, Vector3 v2, Vector3 v3) {
	return (cross(v1,v2) + cross(v2,v3) + cross(v3,v1)).length() * 0.5f;
}

inline Vector3 vmul(const Vector3 &v1, const Vector3 &v2) {
	return Vector3( v1.e[0]*v2.e[0], v1.e[1]*v2.e[1], v1.e[2]*v2.e[2]);
}

typedef std::vector<_Vector4> Vector3List;	
typedef Vector3List::iterator Vector3ListIterator;


#endif //_VECTOR3_H_
