#ifndef _VERTEX_H_
#define _VERTEX_H_

#include "Color.h"
#include "Normal.h"
#include "Vector3.h"

/// A class representing a vertex in 3D space.
class Vertex
{
	public:
		
		/// Construct a default vertex at the origin with texture coordinates (0.0f,0.0f), default color.
		inline Vertex()
			:	pos( Vector3(0.0f, 0.0f, 0.0f) ),
				u( 0.0f ),
				v( 0.0f )
		{
		}
		
		
		/// Construct vertex at the given position with texture coordinates (0.0f,0.0f), default color.
		inline Vertex( float newX, float newY, float newZ )
			:	pos( Vector3(newX, newY, newZ )),
				u( 0.0f ),
				v( 0.0f )
		{
		}
		
		
		/// Construct a vertex with the given position, color, with texture coordinates (0.0f,0.0f).
		inline Vertex( float newX, float newY, float newZ, const Color& newColor )
			:	pos( Vector3(newX, newY, newZ) ),
				color( newColor ),
				u( 0.0f ),
				v( 0.0f )
		{
		}
		
		
		/// Construct a vertex with the given position, color, and texture coordinates.
		inline Vertex( float newX, float newY, float newZ, const Color& newColor, float newU, float newV )
			:	pos( Vector3(newX, newY, newZ) ), 
				color( newColor ),
				u( newU ),
				v( newV )
		{
		}
		
		
		/// Construct a vertex with the given position, color, and texture coordinates.
		inline Vertex( float newX, float newY, float newZ, const Color& newColor,
						float newU, float newV, const Normal& newNormal )
			:	pos(Vector3(newX, newY, newZ)), 
				color( newColor ),
				u( newU ),
				v( newV ),
				normal( newNormal )
		{
		}
		
		
		/// 3D Position of the vertex.
		Vector3 pos;
		
		
		/// Texture coordinates of the vertex.
		float u, v;
		
		
		/// Color of the vertex.
		Color color;
		
		
		/// Normal vector of the vertex.
		Normal normal;
		
		
};

#endif //_VERTEX_H_